94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
100 lines
2.9 KiB
Python
100 lines
2.9 KiB
Python
"""HIP fallback for ``hash_topk``: ``csrc/deepseek_v4/hash_topk.cuh`` uses
|
|
CUDA-only primitives, so on ROCm we dispatch to this Triton implementation.
|
|
"""
|
|
|
|
from __future__ import annotations
|
|
|
|
from typing import Tuple
|
|
|
|
import torch
|
|
import triton
|
|
import triton.language as tl
|
|
|
|
|
|
@triton.jit
|
|
def _hash_topk_triton_kernel(
|
|
router_logits_ptr,
|
|
input_ids_ptr,
|
|
tid2eid_ptr,
|
|
topk_weights_ptr,
|
|
topk_ids_ptr,
|
|
num_routed_experts: tl.constexpr,
|
|
topk_routed: tl.constexpr,
|
|
topk_fused: tl.constexpr,
|
|
routed_scaling_factor,
|
|
BLOCK_K: tl.constexpr,
|
|
):
|
|
token_pos = tl.program_id(0)
|
|
token_id = tl.load(input_ids_ptr + token_pos).to(tl.int64)
|
|
|
|
k_off = tl.arange(0, BLOCK_K)
|
|
routed_mask = k_off < topk_routed
|
|
fused_mask = k_off < topk_fused
|
|
is_shared = k_off >= topk_routed
|
|
|
|
expert_id = tl.load(
|
|
tid2eid_ptr + token_id * topk_routed + k_off,
|
|
mask=routed_mask,
|
|
other=0,
|
|
).to(tl.int32)
|
|
logit = tl.load(
|
|
router_logits_ptr + token_pos * num_routed_experts + expert_id,
|
|
mask=routed_mask,
|
|
other=0.0,
|
|
).to(tl.float32)
|
|
|
|
softplus = tl.maximum(logit, 0.0) + tl.log(1.0 + tl.exp(-tl.abs(logit)))
|
|
weight = tl.sqrt(softplus)
|
|
weight = tl.where(routed_mask, weight, 0.0)
|
|
routed_sum = tl.sum(weight, axis=0)
|
|
|
|
shared_weight = 1.0 / routed_scaling_factor
|
|
final_weight = tl.where(is_shared, shared_weight, weight / routed_sum)
|
|
shared_id = num_routed_experts + (k_off - topk_routed)
|
|
final_id = tl.where(is_shared, shared_id, expert_id).to(tl.int32)
|
|
|
|
out_off = token_pos * topk_fused + k_off
|
|
tl.store(topk_weights_ptr + out_off, final_weight, mask=fused_mask)
|
|
tl.store(topk_ids_ptr + out_off, final_id, mask=fused_mask)
|
|
|
|
|
|
def hash_topk_triton(
|
|
router_logits: torch.Tensor,
|
|
input_ids: torch.Tensor,
|
|
tid2eid: torch.Tensor,
|
|
num_fused_shared_experts: int,
|
|
routed_scaling_factor: float,
|
|
scoring_func: str,
|
|
) -> Tuple[torch.Tensor, torch.Tensor]:
|
|
assert scoring_func == "sqrtsoftplus"
|
|
|
|
num_tokens = router_logits.size(0)
|
|
num_routed_experts = router_logits.size(1)
|
|
topk_routed = tid2eid.size(1)
|
|
topk_fused = topk_routed + num_fused_shared_experts
|
|
|
|
topk_weights = torch.empty(
|
|
(num_tokens, topk_fused), dtype=torch.float32, device=router_logits.device
|
|
)
|
|
topk_ids = torch.empty(
|
|
(num_tokens, topk_fused), dtype=torch.int32, device=router_logits.device
|
|
)
|
|
if num_tokens == 0:
|
|
return topk_weights, topk_ids
|
|
|
|
block_k = max(triton.next_power_of_2(topk_fused), 1)
|
|
_hash_topk_triton_kernel[(num_tokens,)](
|
|
router_logits,
|
|
input_ids,
|
|
tid2eid,
|
|
topk_weights,
|
|
topk_ids,
|
|
num_routed_experts=num_routed_experts,
|
|
topk_routed=topk_routed,
|
|
topk_fused=topk_fused,
|
|
routed_scaling_factor=float(routed_scaling_factor),
|
|
BLOCK_K=block_k,
|
|
num_warps=1,
|
|
)
|
|
return topk_weights, topk_ids
|